//
//
//    xorgame - some XOR brick game
//    Copyright (C) 1999  Florian Berger
//    Email: florian.berger@jk.uni-linz.ac.at
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License Version 2 as
//    published by the Free Software Foundation;
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//


//#include <iostream.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#include <unistd.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qscrollbar.h>
#include <qkeycode.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qimage.h>
#include <qtimer.h>
#include <qlcdnumber.h>
#include <qbitmap.h>
#include <math.h>


#define NO  FALSE
#define YES TRUE

#define Abs(x) (((x)>0)?(x):(-x))

//inline int Abs( int x ) { return( (x>0)?x:-x ); }



inline double rnd_01(void)
{return( (double)(rand()&0xFFFFFF)/(double)(0x1000000));}

inline double rnd(double nr){ return(rnd_01()*nr); }

void randomize(void)
{
    srand(time(NULL));
}


class MainWidget : public QWidget 
{
 Q_OBJECT
 public:
   MainWidget( QWidget *parent, char *name , int argc, char **argv );
   ~MainWidget();
   int init_audio();
   int sampnr;
   int sampfreq;
   int blksize;
   int audio;
   int xm,ym;
   int c_anz;
   int c_x[100],c_y[100];
   int k_anz;
   int k_x[100],k_y[100];
   int x_anz,y_anz;
   int def_u_key;
   int def_d_key;
   int def_l_key;
   int def_r_key;
   int def_f_key;
   int def_s_key;
   char scene[81][81];
   int keypressed;
   int score;
   int x_pg,y_pg;
   QPixmap c_pic;
   QPixmap m_pic;
   QPixmap k_pic;
   QPixmap b_pic;
   QPixmap h_pic;
   QPixmap all_pic;
   int poss_x[8];
   int poss_y[8];
   int levelnr;
   struct Stone{
       int nr;
       int xy[20];
   } stone;
 protected:
     void paintEvent(QPaintEvent *) { printscene(); }
//   void resizeEvent(QResizeEvent *);
   void keyPressEvent(QKeyEvent *);
 private:

    
     void draw_stone(int x, int y, QPainter *p)
     {
         int j,xx,yy;
         for(j=0;j<stone.nr;j++){
             xx=x+stone.xy[ j*2 ];
             yy=y+stone.xy[j*2+1];
             if( xx>0 && xx<x_anz-1 && yy>0 && yy<y_anz-1 )
                 p->drawPixmap( xx*b_pic.width(), yy*b_pic.height(),  m_pic);
         }
     }


     void put_stone(int x, int y)
     {
         int j;
         char X,X2;
         
         for(j=0;j<stone.nr;j++){
             X=scene[ y+stone.xy[j*2+1] ][ x+stone.xy[j*2]   ];
             X2=X;
             if(X=='B') X2='.';
             if(X=='.') X2='B'; 
             scene[ y+stone.xy[j*2+1] ][ x+stone.xy[j*2]   ]=X2;
         }
     }

     
     void load_level(int nr)
     {
         int i,j,reset;
         static int pos[500];
         y_anz=5+2;
         x_anz=5+2;
         if (nr<0) { nr=-nr; reset=1; } else reset=0;

         randomize();

         stone.nr=5;
         
         stone.xy[0]= 0;  stone.xy[1]= 0;
         stone.xy[2]= 1;  stone.xy[3]= 0;
         stone.xy[4]= 0;  stone.xy[5]= 1;
         stone.xy[6]=-1;  stone.xy[7]= 0;
         stone.xy[8]= 0;  stone.xy[9]=-1;
         
         for(j=0;j<y_anz;j++) for(i=0;i<x_anz;i++) scene[j][i]='.';

         for(i=0;i<nr;i++){
             do{
                 if (! reset) pos[i]=(int)rnd((x_anz-2)*(y_anz-2));
                 for(j=0;pos[i]!=pos[j] && j<i;j++);
             }while( j!=i );
             printf("X=%d,Y=%d\n",1+pos[i]%(x_anz-2),1+pos[i]/(x_anz-2));
             put_stone(1+pos[i]%(x_anz-2),1+pos[i]/(x_anz-2));
         }
         
         for(i=0;i<x_anz;i++){
             scene[   0   ][i]='H';
             scene[y_anz-1][i]='H';
         }
         for(i=0;i<y_anz;i++){
             scene[i][   0   ]='H';
             scene[i][x_anz-1]='H';
             scene[i][ x_anz ]='0';
         }
             
         
	 c_anz=0;
         k_anz=0;
         if( xm<1 || xm>x_anz-2 || ym<1 || ym>y_anz-2 ){
             xm=x_anz/2; ym=y_anz/2;
         }
     }
   
   void load_next_level(void)
     {
	levelnr++;
	load_level(levelnr);
     }
   
   void recalcscene()
     {int i,ready;
      double dmx,dmy,ddm,prob;
      double rm=1.2;
      
        ready=1;
	for(i=0;i<(x_anz-2)*(y_anz-2);i++){
            if( scene[1+i/(x_anz-2)][1+i%(x_anz-2)] == 'B' ){ ready=0;break; }
	}
	if( ready ) load_next_level();
	printscene();
     }
   
   void setscene(int x, int y, char c)
     {
	scene[y][x]=c;
     }
   
   void mouse_right()
     {
	xm++; if (xm>x_anz-2) xm=x_anz-2;
	recalcscene();
     }

   void mouse_left()
     {
	xm--; if (xm<1) xm=1;
	recalcscene();
     }

   void mouse_down()
     {
	ym++; if (ym>y_anz-2) ym=y_anz-2;
	recalcscene();
     }

   void mouse_up()
     {
	ym--; if (ym<1) ym=1;
	recalcscene();
     }

     
   void printscene()
     {int i,j;
      char str[80];
        QPainter p(&all_pic);
	//printf("Score: %d\n",score);
	p.eraseRect(all_pic.rect());
	for(i=0;i<y_anz;i++){
	   //printf("%s\n",scene[i]);
	   for(j=0;j<x_anz;j++){
	      switch (scene[i][j]){
	       case 'B':
		 p.drawPixmap(j*b_pic.width(),i*b_pic.height(),b_pic); break;
	       case 'H':
		 p.drawPixmap(j*b_pic.width(),i*b_pic.height(),h_pic); break;
	       case 'C':
		 p.drawPixmap(j*b_pic.width(),i*b_pic.height(),c_pic); break;
	       case 'K':
		 p.drawPixmap(j*b_pic.width(),i*b_pic.height(),k_pic); break;
	      }
	   }
	}

        draw_stone(xm,ym,&p);
        
	p.end();
	p.begin(this);
	p.drawPixmap(0,20,all_pic);
	p.eraseRect(0,0,all_pic.width(),20);
//	p.drawPixmap(2,2,k_pic);
	sprintf(str,"sc:%05d",score);
	p.drawText(2,2+16,str);
	sprintf(str,"lev:%2d",levelnr);
	p.drawText(2+16+40,2+16,str);
	p.end();
     }

// private slots:
   
};



int MainWidget::init_audio()
{
   int audio = open("/dev/dsp", O_RDONLY, 0);
   if (audio == -1) {
      perror("/dev/dsp");
      exit(1);
   }
   
//   blksize=16;
//   ioctl(audio, SNDCTL_DSP_SETFRAGMENT, &blksize);
   
   ioctl(audio, SNDCTL_DSP_GETBLKSIZE, &blksize);
   blksize=16;
   
   ioctl(audio, SNDCTL_DSP_SYNC, NULL);
   int a_sampsize = 8;
   ioctl(audio, SNDCTL_DSP_SAMPLESIZE, &a_sampsize);
   
   int a_stereo = 0;
   ioctl(audio, SNDCTL_DSP_STEREO, &a_stereo);
   
   int a_speed = sampfreq;
   printf("sampfreq=%d\n",sampfreq);
   ioctl(audio, SNDCTL_DSP_SPEED, &a_speed);
   ioctl(audio, SOUND_PCM_READ_RATE, &sampfreq);
   printf("sampfreq=%d\n",sampfreq);

//   int mixer = open("/dev/mixer", O_RDONLY, 0);
//   int vol=100;
//   ioctl(mixer, SOUND_MIXER_WRITE_MIC, &vol);
//   close(mixer);
   
   return(audio);
}




MainWidget::MainWidget( QWidget *parent, char *name, int argc, char **argv )
               : QWidget(parent,name)
{int i;
   //sampnr=1024;
   //sampfreq=10989;
   //audio = init_audio();
   //printf("Audiodriver initialized\n");
   
   c_pic.load("cat.xpm");
   m_pic.load("mouse_l.xpm");
   k_pic.load("cheese_l.xpm");
   b_pic.load("block_l.xpm");
   h_pic.load("hardblock_l.xpm");

   def_u_key = Key_Up;
   def_d_key = Key_Down;
   def_r_key = Key_Right;
   def_l_key = Key_Left;
   def_f_key = Key_Return;
   def_s_key = Key_Escape;
   
   poss_x[0]=-1; poss_y[0]=-1;
   poss_x[1]= 0; poss_y[1]=-1;
   poss_x[2]=+1; poss_y[2]=-1;
   poss_x[3]=+1; poss_y[3]= 0;
   poss_x[4]=+1; poss_y[4]=+1;
   poss_x[5]= 0; poss_y[5]=+1;
   poss_x[6]=-1; poss_y[6]=+1;
   poss_x[7]=-1; poss_y[7]= 0;
   
   keypressed=0;
   score=0;
   levelnr=-1;
   
   x_pg=0; y_pg=20;

   load_next_level();
   
   if (argc>1) load_level(argv[1][0]-48);
   
//   setGeometry(10,10,16*x_anz,16*y_anz+20);
   resize(b_pic.width()*x_anz,b_pic.height()*y_anz+20);
   all_pic.resize(b_pic.width()*x_anz,b_pic.height()*y_anz);

//   recalcscene();
   
//   QTimer *timer = new QTimer(this);
//   connect( timer, SIGNAL(timeout()), this, SLOT(timer_proc()) );
//   timer->start(400);
}



//void MainWidget::paintEvent( QPaintEvent * )
//{
//}



//void MainWidget::resizeEvent( QResizeEvent * )
//{
//}



void MainWidget::keyPressEvent(QKeyEvent *k)
{int key,i;
    key = k->key();
    if (key==def_r_key) mouse_right();
    else
    if (key==def_l_key) mouse_left();
    else
    if (key==def_u_key) mouse_up();
    else
    if (key==def_d_key) mouse_down();
    else
    if (key==def_f_key) {put_stone(xm,ym); recalcscene();}
    else
    if (key==def_s_key) {load_level(-levelnr); recalcscene();}
    else
    k->ignore();

}



MainWidget::~MainWidget()
{
   close(audio);
}





main(int argc, char *argv[])
{
   QApplication a( argc,argv );
//   QApplication::setFont( QFont("OldEnglish") );
   MainWidget mw( 0, "MainWidget", argc, argv );
   a.setMainWidget( &mw );
   mw.show();
   return(a.exec());
}

#include"moc_xorgame.cpp"
